﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace TanyaLab
{
    public partial class Form1 : Form
    {
        private Bitmap bitmap;
        private Graphics g;

        private int width, height; //ширина и высота области рисования
        private float scale; //коэффициент масштабирования
        private float translateX, translateY; //смещение по х и у

        private SolidBrush brush; //служит для закрашивания многоугольника
        private Random random; //служит для выбора случайных цветов

        private Polygon polygon;

        public Form1()
        {
            InitializeComponent();
        }

        private void draw()
        {
            g.Clear(Color.White);
            polygon.fill(g, brush, translateX, translateY, scale);
        }

        private void pictureBox_MouseClick(object sender, MouseEventArgs e)
        {
            //так как наш Polygon проверяет попадания без учета translate и scale, преобразуем х и у координаты
            float x = (e.X - translateX) / scale + polygon.MinX();
            float y = (e.Y - translateY) / scale + polygon.MinY();

            Point p = new Point((int)x, (int)y);

            //изменяем цвет многоугольника при попадании
            if (touch(polygon, p))
            {
                brush.Color = randomColor();
                draw();
                pictureBox.Invalidate();
            }
        }

        private bool touch(Polygon polygon, Point a) //проверка попадания точки в Polygon
        {
            Polygon.PointInPolygon touch = polygon.pointInPolygon(a);
            return touch == Polygon.PointInPolygon.INSIDE || touch == Polygon.PointInPolygon.BOUNDARY;
        }

        private Color randomColor() //возвращает случайный цвет
        {
            if (random == null)
                random = new Random();
            return Color.FromArgb(random.Next(25, 225), random.Next(25, 225), random.Next(25, 225)); //случайный цвет
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            bitmap = new Bitmap(pictureBox.Width, pictureBox.Height);
            pictureBox.Image = bitmap;

            g = Graphics.FromImage(bitmap);
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;

            width = bitmap.Width;
            height = bitmap.Height;

            //выбираем случайный цвет закрашивания
            brush = new SolidBrush(randomColor());

            //создаем многоугольник
            int[] vertex = new int[]
            {
                370, 279,
                160, 106,
                262, 390,
                61, 433,
                358, 530,
                160, 869,
                412, 624,
                439, 942,
                528, 601,
                672, 797,
                598, 425,
                673, 276,
                427, 420,
                660, 124,
                358, 356,
                507, 95
            }; //х и y координаты многоугольника против часовой стрелки
            polygon = new Polygon(vertex);

            //растягимаем многоугольник на всю область рисования и устанавливаем в центре
            scale = Math.Min(pictureBox.Width / (float)polygon.width(), pictureBox.Height / (float)polygon.height());
            translateX = (pictureBox.Width - polygon.width() * scale) / 2;
            translateY = (pictureBox.Height - polygon.height() * scale) / 2;

            draw();
        }


    }
}
